﻿using System;
using UnityEngine;
using System.Collections;
using System.Collections.Generic;
using SimpleJSON;
using System.Linq;
using AssetBundles;
using UnityEngine.UI;

namespace Feel.common
{
    public class Calculation : MVRScript
    {
        private EventTrigger _calculation_event;
        private JSONStorableFloat _drainage;
        private JSONStorableFloat _input_value;

        public override void Init()
        {
            try
            {
                _input_value = new JSONStorableFloat("value", 0f, OnInputValue, 0f, 1f);
                RegisterFloat(_input_value);

                UIDynamicButton calculation_btn = CreateButton("Calculation Output", false);
                calculation_btn.button.onClick.AddListener(OnMovieEventFunc);

                _drainage = new JSONStorableFloat("drainage", 1f, (float value) =>
                {
                    OnInputValue(_input_value.val);
                }, 0f, 10f);
                RegisterFloat(_drainage);
                CreateSlider(_drainage, false);

                CCEventHandler.LoadAssets();

                // A tiny bit of help to give some info to your fellow creator and/or player
                JSONStorableString helpText = new JSONStorableString("About",
                    "<color=#000><size=35><b>Calculation</b></size></color>\n\n" +
                    "<color=#333>" +
                    "Supports simple company-side calculations.\n\n" +
                    "thank you for your support\n" +
                    "<b>https://www.patreon.com/feel_vam</b>\n\n" +
                    "</color>");
                UIDynamic helpTextfield = CreateTextField(helpText, true);
                helpTextfield.height = 300.0f;

                var btn = CreateButton("Go To Support", true);
                btn.button.onClick.AddListener(() =>
                {
                    Application.OpenURL("https://www.patreon.com/feel_vam");
                });

            }
            catch (Exception e)
            {
                SuperController.LogError("Calculation.cs Exception caught: " + e);
            }
        }

        private void OnInputValue(float value)
        {
            if (_calculation_event != null) _calculation_event.TriggerTransition(_input_value.val * _drainage.val);
        }

        private void OnMovieEventFunc()
        {
            if (_calculation_event == null) _calculation_event = new EventTrigger(this, "Calculation Event");
            _calculation_event.OpenTriggerTransitionActionsPanel();
        }

        public override JSONClass GetJSON(bool includePhysical = true, bool includeAppearance = true, bool forceStore = false)
        {
            JSONClass jc = base.GetJSON(includePhysical, includeAppearance, forceStore);
            if (includePhysical || forceStore)
            {
                needsStore = true;
                if (_calculation_event != null) jc[_calculation_event.Name] = _calculation_event.GetJSON(base.subScenePrefix);
            }
            return jc;
        }

        public override void RestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, JSONArray presetAtoms = null, bool setMissingToDefault = true)
        {
            base.RestoreFromJSON(jc, restorePhysical, restoreAppearance, presetAtoms, setMissingToDefault);

            //_movie_start_event = new EventTrigger(this, "Movie Start Event");
            if (jc.HasKey("Calculation Event")) _calculation_event = new EventTrigger(this, "Calculation Event");

            //SuperController.LogMessage(">> 1 RestoreFromJSON " + jc.ToString());

            base.RestoreFromJSON(jc, restorePhysical, restoreAppearance, presetAtoms, setMissingToDefault);
        }

        public override void LateRestoreFromJSON(JSONClass jc, bool restorePhysical = true, bool restoreAppearance = true, bool setMissingToDefault = true)
        {
            base.LateRestoreFromJSON(jc, restorePhysical, restoreAppearance, setMissingToDefault);

            if (!base.physicalLocked && restorePhysical && !IsCustomPhysicalParamLocked("trigger"))
            {
                if (jc.HasKey("Calculation Event")) _calculation_event.RestoreFromJSON(jc, base.subScenePrefix, base.mergeRestore, setMissingToDefault);
            }
        }

        public class EventTrigger : Trigger
        {
            public MVRScript Owner
            {
                get; private set;
            }

            private string _name;

            public string Name
            {
                get { return _name; }
            }

            public EventTrigger(MVRScript owner, string name)
            {
                Owner = owner;
                _name = name;
                handler = CCEventHandler.Instance;
            }

            public void OpenPanelActionStart()
            {
                if (!CCEventHandler.Loaded)
                {
                    SuperController.LogError("EventTrigger: You need to call CCEventHandler.LoadAssets() before use.");
                    return;
                }

                triggerActionsParent = Owner.UITransform;
                InitTriggerUI();
                OpenTriggerActionsPanel();

                triggerActionsPanel.Find("Panel/Header Text").GetComponent<Text>().text = _name;
                triggerActionsPanel.Find("Content/Tab1/Label").GetComponent<Text>().text = "Event Actions";
                triggerActionsPanel.Find("Content/Tab2").gameObject.SetActive(false);
                triggerActionsPanel.Find("Content/Tab3").gameObject.SetActive(false);
            }

            public void OpenTriggerTransitionActionsPanel()
            {
                if (!CCEventHandler.Loaded)
                {
                    SuperController.LogError("EventTrigger: You need to call SimpleTriggerHandler.LoadAssets() before use.");
                    return;
                }

                triggerActionsParent = Owner.UITransform;
                InitTriggerUI();
                OpenTriggerActionsPanel();

                triggerActionsPanel.Find("Panel/Header Text").GetComponent<Text>().text = Name;
                triggerActionsPanel.Find("Content/Tab1").gameObject.SetActive(false);
                triggerActionsPanel.Find("Content/Tab2").gameObject.SetActive(true);
                triggerActionsPanel.Find("Content/Tab2").GetComponent<Toggle>().isOn = true;
                triggerActionsPanel.Find("Content/Tab3").gameObject.SetActive(false);

            }

            public void RestoreFromJSON(JSONClass jc, string subScenePrefix, bool isMerge, bool setMissingToDefault)
            {
                if (jc.HasKey(_name))
                {
                    JSONClass tc = jc[_name].AsObject;
                    if (tc != null)
                        base.RestoreFromJSON(tc, subScenePrefix, isMerge);
                }
                else if (setMissingToDefault)
                {
                    base.RestoreFromJSON(new JSONClass());
                }
            }

            public void Trigger()
            {
                active = true;
                active = false;
            }

            public void TriggerTransition(float val)
            {
                active = true;
                this.transitionInterpValue = val;

                active = false;
            }
        }
    }

    public class CCEventHandler : TriggerHandler
    {
        public static bool Loaded { get; private set; }

        private static CCEventHandler myInstance;

        private RectTransform myTriggerActionsPrefab;
        private RectTransform myTriggerActionMiniPrefab;
        private RectTransform myTriggerActionDiscretePrefab;
        private RectTransform myTriggerActionTransitionPrefab;

        public static CCEventHandler Instance
        {
            get
            {
                if (myInstance == null)
                    myInstance = new CCEventHandler();
                return myInstance;
            }
        }

        public static void LoadAssets()
        {
            SuperController.singleton.StartCoroutine(Instance.LoadAssetsInternal());
        }

        private IEnumerator LoadAssetsInternal()
        {
            foreach (var x in LoadAsset("z_ui2", "TriggerActionsPanel", p => myTriggerActionsPrefab = p))
                yield return x;
            foreach (var x in LoadAsset("z_ui2", "TriggerActionMiniPanel", p => myTriggerActionMiniPrefab = p))
                yield return x;
            foreach (var x in LoadAsset("z_ui2", "TriggerActionDiscretePanel", p => myTriggerActionDiscretePrefab = p))
                yield return x;
            foreach (var x in LoadAsset("z_ui2", "TriggerActionTransitionPanel", p => myTriggerActionTransitionPrefab = p))
                yield return x;

            Loaded = true;
        }

        private IEnumerable LoadAsset(string assetBundleName, string assetName, Action<RectTransform> assign)
        {
            AssetBundleLoadAssetOperation request = AssetBundleManager.LoadAssetAsync(assetBundleName, assetName, typeof(GameObject));
            if (request == null)
                throw new NullReferenceException($"Request for {assetName} in {assetBundleName} assetbundle failed: Null request.");
            yield return request;
            GameObject go = request.GetAsset<GameObject>();
            if (go == null)
                throw new NullReferenceException($"Request for {assetName} in {assetBundleName} assetbundle failed: Null GameObject.");
            RectTransform prefab = go.GetComponent<RectTransform>();
            if (prefab == null)
                throw new NullReferenceException($"Request for {assetName} in {assetBundleName} assetbundle failed: Null RectTansform.");
            assign(prefab);
        }


        void TriggerHandler.RemoveTrigger(Trigger t)
        {
            // nothing to do
        }

        void TriggerHandler.DuplicateTrigger(Trigger t)
        {
            throw new NotImplementedException();
        }

        RectTransform TriggerHandler.CreateTriggerActionsUI()
        {
            return UnityEngine.Object.Instantiate(myTriggerActionsPrefab);
        }

        RectTransform TriggerHandler.CreateTriggerActionMiniUI()
        {
            return UnityEngine.Object.Instantiate(myTriggerActionMiniPrefab);
        }

        RectTransform TriggerHandler.CreateTriggerActionDiscreteUI()
        {
            return UnityEngine.Object.Instantiate(myTriggerActionDiscretePrefab);
        }

        RectTransform TriggerHandler.CreateTriggerActionTransitionUI()
        {
            return UnityEngine.Object.Instantiate(myTriggerActionTransitionPrefab);
        }

        void TriggerHandler.RemoveTriggerActionUI(RectTransform rt)
        {
            UnityEngine.Object.Destroy(rt?.gameObject);
        }
    }
}
